from Globals import InitializeClass
from Products.CMFCore.utils import getToolByName
from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import StringField
from Products.Archetypes.Registry import registerField
from AccessControl import AuthEncoding


def get_scheme_id(pw):
    for id, prefix, scheme in AuthEncoding._schemes:
        lp = len(prefix)
        if pw[:lp] == prefix:
            return id
    return None

class PasswordField(StringField):
    """
    Handles storage of encripted password.
    """

    __implements__ = StringField.__implements__

    _properties = StringField._properties.copy()
    _properties.update({
        'encryption': None,
        'length': 6,
        'fixed_length': False,
        'auto_generate': False,
        'validators': ('PasswordMatch',),
    })

    security = ClassSecurityInfo()

    security.declarePrivate('validateAttempt')
    def validateAttempt(self, instance, attempt):
        reference = self.getRaw(instance)
        return AuthEncoding.pw_validate(reference, attempt)

    security.declarePrivate('getEncryption')
    def getEncryption(self, instance):
        """
        Get the encryption of the stored value (LDAP-style encoding). 
        If no value is stored, return field's default encondig.
        """
        value = self.getRaw(instance)
        if value:
            return get_scheme_id(value)
        return field.encryption

    security.declarePrivate('autoGenerate')
    def autoGenerate(self, instance, **kwargs):
        """
        Auto-generates the password. Returns the generated value. Sets
        the value using mutator.
        """
        reg_tool = getToolByName(instance, 'portal_registration')
        value = reg_tool.getPassword(length=self.length)
        return value

    def isEncrypted(self):
        return not not self.encryption

    security.declarePrivate('set')
    def set(self, instance, value, **kwargs):
        if type(value) == type(tuple()):
            value = value[0]
        if value and self.encryption:
            hash = AuthEncoding.pw_encrypt(value, encoding=self.encryption)
        else:
            hash = value
        StringField.set(self, instance, hash, **kwargs)


    security.declarePrivate('getDefault')
    def getDefault(self, instance):
        """
        Return the default value to be used in initalization. If
        'auto_generate' is true, generates a password. But it still 
        respects the 'default' and 'default_method' properties, relying 
        on the implementation of the base class.
        """
        default_value = StringField.getDefault(self, instance)
        if self.auto_generate and not (default_value or self.isEncrypted()):
            # Only perform auto-generation if no encryption or default value
            # ar set.
            return self.autoGenerate(instance)
        return default_value



registerField(PasswordField, title='Password',
    description="Used to store encripted passwords.")
