# vim: set ts=4 sw=4 et:
"""
Test suite for module utils.
"""

import os, sys
from Testing import ZopeTestCase

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Globals import package_home
PACKAGE_HOME = package_home(globals())
from os.path import join

from Acquisition import aq_base

from Products.PloneTestCase import PloneTestCase
from AccessControl import Unauthorized
from Products.PasswordField import config
from Products.PortalTransforms.transforms.office_wvware import document

ZopeTestCase.installProduct('Archetypes')
ZopeTestCase.installProduct('MimetypesRegistry')
ZopeTestCase.installProduct('PortalTransforms')
ZopeTestCase.installProduct('PasswordField')


PLONE_PRODUCTS = (
    'Archetypes',
    'MimetypesRegistry',
    'PortalTransforms',
    'PasswordField',
)
PloneTestCase.setupPloneSite(products=PLONE_PRODUCTS)


class TestPasswordField(PloneTestCase.PloneTestCase):
    """
    Test suite for PasswordField.
    """

    def afterSetUp(self):
        PloneTestCase.PloneTestCase.afterSetUp(self)
        self.folder.invokeFactory(type_name='SampleContent', id='sample1')

    def test_contentCreation(self):
        self.failUnless(hasattr(aq_base(self.folder), 'sample1'))

    def test_attempt(self):
        # Test attempt
        obj = getattr(self.folder, 'sample1')
        obj.setPassword('qwerty')
        field = obj.getField('password')
        self.failUnless(field.validateAttempt(obj, 'qwerty'))

        # Test password generation and attempt
        field.auto_generate = 1
        field.encryption = None
        self.folder.invokeFactory(type_name='SampleContent', id='sample2')
        obj2 = getattr(self.folder, 'sample2')
        self.failUnless(obj2.getPassword())

        # Testing with some encryption setted. Shouldn't auto-generate.
        field.encryption = 'SHA'
        self.folder.invokeFactory(type_name='SampleContent', id='sample3')
        obj3 = getattr(self.folder, 'sample3')
        self.failIf(aq_base(obj3).getPassword())

    def test_get_encryption(self):
        obj = getattr(self.folder, 'sample1')
        field = obj.getField('password')
        field.encryption = 'SSHA'
        obj.setPassword('qwerty')
        field = obj.getField('password')
        self.assertEqual(field.getEncryption(obj), 'SSHA')

        # Changing encryption
        field.encryption = 'MYSQL'
        self.assertEqual(field.getEncryption(obj), 'SSHA')
        obj.setPassword('qwerty')
        self.assertEqual(field.getEncryption(obj), 'MYSQL')

 
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestPasswordField))
    return suite

if __name__ == '__main__':
    framework()

