from Products.validation import validation
from Products.validation.interfaces.IValidator import IValidator
from Products.CMFCore.utils import getToolByName

class PasswordValidator:
    """
    Password validator. To be used with PasswordField.
    """

    __implements__ = IValidator

    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title
        self.description = description


    def __call__(self, value, *args, **kwargs):
        """
        Tests if the password values match. If value is just one string, 
        only tests size.
        """

        if type(value) == type(tuple()):
            passwd, confirm_passwd = value
            if passwd != confirm_passwd:
                return 'Password mismatch.'
            value = passwd

        field = kwargs.get('field', None)

        # Test  value's length
        if len(value) < field.length:
            return 'Password size invalid. Password must contain '\
                'at least %d characters.' % field.length

        if field.fixed_length and len(value) != field.length:
            return 'Password size invalid. Password must contain '\
                'exactly %d characters.' % field.length

        return 1



password_validator = PasswordValidator('PasswordMatch', 
    'Password matches confirmation')

validation.register(password_validator)
